import contextlib
import mysql.connector


equipment_location_query = "SELECT " \
                        "  e.absid, " \
                        "  e.name AS equipment_name, " \
                        "  p.name AS pop_name, " \
                        "  p.abbreviation AS pop_abbreviation, " \
                        "  p.site_id AS pop_site_id, " \
                        "  p.country, " \
                        "  g.longitude, " \
                        "  g.latitude " \
                        "FROM " \
                        "  equipment e " \
                        "INNER JOIN pop p " \
                        "  ON p.absid = e.PTR_pop " \
                        "INNER JOIN geocoding g " \
                        "  ON g.absid = p.PTR_geocoding " \
                        "WHERE " \
                        "  e.status != 'terminated' " \
                        "  AND e.status != 'disposed'"


circuit_hierarchy_query = "SELECT " \
                     "  pc.name AS parent_circuit, " \
                     "  pc.absid AS parent_circuit_id, " \
                     "  pc.status AS parent_circuit_status, " \
                     "  cc.name AS child_circuit, " \
                     "  cc.absid AS child_circuit_id, " \
                     "  cc.status AS child_circuit_status, " \
                     "  cg.segment_group AS segment_group " \
                     "FROM circuit_glue cg " \
                     "INNER JOIN circuit pc ON pc.absid = cg.PTR_circuit " \
                     "INNER JOIN circuit cc ON cc.absid = cg.PTR_component"


retrieve_services_query_template = "SELECT " \
                          "  c.absid AS id, " \
                          "  c.name, " \
                          "  c.status, " \
                          "  c.circuit_type, " \
                          "  c.service_type, " \
                          "  events.short_descr AS project, " \
                          "  e.name AS equipment, " \
                          "  cc.{} AS port, " \
                          "  cc.{} AS logical_unit, " \
                          "  LOWER(o.name) AS manufacturer, " \
                          "  LOWER(ec.card_id) AS card_id, " \
                          "  LOWER(IF(pp.interface_name IS NULL," \
                                   " ''," \
                                   " pp.interface_name))" \
                                   " AS interface_name " \
                          "FROM circuit c " \
                          "INNER JOIN circuit_connections cc " \
                          "  ON cc.circ_absid = c.absid " \
                          "INNER JOIN equipment e " \
                          "  ON e.absid = cc.{} " \
                          "LEFT JOIN events " \
                          "  ON events.absid = cc.PTR_project " \
                          "INNER JOIN equipment_card ec " \
                          "  ON ec.absid = cc.{} " \
                          "LEFT JOIN organisation o " \
                          "  ON o.absid = ec.manufacturer " \
                          "LEFT JOIN port_plugin pp " \
                          "  ON pp.PTR_card = cc.{} " \
                          "  AND pp.port = cc.{} " \
                          "WHERE " \
                          "  c.status  != 'terminated' " \
                          "  AND is_circuit = 1 "

order_services_outer_query = "SELECT * FROM ({}) AS inner_query " \
                 " ORDER BY FIELD(status, 'spare', 'planned'," \
                 " 'ordered', 'installed', 'operational')"


connection_variants = [
    ['port_a',
     'int_LU_a',
     'PTR_equip_a',
     'PTR_card_a',
     'PTR_card_a',
     'port_a'],
    ['port_b',
     'int_LU_b',
     'PTR_equip_b',
     'PTR_card_b',
     'PTR_card_b',
     'port_b'],
    ['port_a_OUT',
     'int_LU_a',
     'PTR_equip_a',
     'PTR_card_a_OUT',
     'PTR_card_a_OUT',
     'port_a_OUT'],
    ['port_b_OUT',
     'int_LU_b',
     'PTR_equip_b',
     'PTR_card_b_OUT',
     'PTR_card_b_OUT',
     'port_b_OUT']
]


@contextlib.contextmanager
def connection(opsdb):  # pragma: no cover
    cx = None
    try:
        cx = mysql.connector.connect(
            host=opsdb["hostname"],
            user=opsdb["username"],
            passwd=opsdb["password"],
            db=opsdb["dbname"])
        yield cx
    finally:
        if cx:
            cx.close()


@contextlib.contextmanager
def cursor(cnx):  # pragma: no cover
    csr = None
    try:
        csr = cnx.cursor()
        yield csr
    finally:
        if csr:
            csr.close()


def _convert_to_dict(crs):
    return [dict((crs.description[i][0], "" if value is None else value)
                 for i, value in enumerate(row)) for row in crs.fetchall()]


def _infinera_field_update(record):
    equipment_parts = record["equipment"].rsplit("-", 1)
    card_parts = record["card_id"].split("-", 1)
    try:
        record["equipment"] = equipment_parts[0]
        record["interface_name"] = equipment_parts[1] + "-" + card_parts[1]
    except IndexError:
        pass  # Nothing to see here
    if record["port"] is not None and record["port"] != "":
        record["interface_name"] += "-" + record["port"]
    record["interface_name"] = record["interface_name"]\
        .replace("--", "-").upper()
    return record


def _juniper_field_update(record):
    if not record["interface_name"]:
        record["interface_name"] = record["card_id"]
        if record["port"] is not None and record["port"] != "":
            separator = "/" if "-" in record["interface_name"] else ""
            record["interface_name"] += separator + record["port"]
    if record["logical_unit"] is not None and record["logical_unit"] != "":
        record["interface_name"] += "." + str(record["logical_unit"])
    return record


def _update_fields(r):
    func = globals().get("_" + r["manufacturer"] + "_field_update")
    return func(r) if func else r


def get_circuits(db):
    retrieve_services_query = _generate_get_circuits_sql()
    with cursor(db) as crs:
        crs.execute(retrieve_services_query)
        r = _convert_to_dict(crs)
    r = list(map(_update_fields, r))
    return r


def _generate_get_circuits_sql():
    retrieve_services_inner_query = " UNION ".join(
        map(lambda e: retrieve_services_query_template.format(*e),
            connection_variants))
    retrieve_services_query = order_services_outer_query.\
        format(retrieve_services_inner_query)
    return retrieve_services_query


def get_circuit_hierarchy(db):
    with cursor(db) as crs:
        crs.execute(circuit_hierarchy_query)
        r = _convert_to_dict(crs)
    return r


def get_equipment_location_data(db):
    with cursor(db) as crs:
        crs.execute(equipment_location_query)
        r = _convert_to_dict(crs)
    return r

