import functools
import json

from flask import Blueprint, request, Response, current_app
from inventory_provider import opsdb
from inventory_provider import db

routes = Blueprint("inventory-opsdb-query-routes", __name__)

services_key_template = "inv_services::{}"
interfaces_key_template = "inv_interfaces::{}::{}"
equipment_locations_key_template = "inv_eq_locations::{}"

services_key = "inv_services"
interfaces_key = "inv_interfaces"
equipment_locations_key = "inv_eq_locations"


def require_accepts_json(f):
    """
    used as a route handler decorator to return an error
    unless the request allows responses with type "application/json"
    :param f: the function to be decorated
    :return: the decorated function
    """
    @functools.wraps(f)
    def decorated_function(*args, **kwargs):
        # TODO: use best_match to disallow */* ...?
        if not request.accept_mimetypes.accept_json:
            return Response(
                response="response will be json",
                status=406,
                mimetype="text/html")
        return f(*args, **kwargs)
    return decorated_function


def _decode_utf8_dict(d):
    return {k.decode('utf8'): json.loads(v) for k, v in d.items()}


@routes.route("/circuit-hierarchy", methods=['GET', ])
@require_accepts_json
def opsdb_circuit_hierarchy():
    #  todo - Retrieve data from redis
    result = [{"Parent Name": "circuit 1"}, {"Parent Name": "Circuit 2"}]

    return Response(
        json.dumps(result),
        mimetype="application/json")


@routes.route("/interfaces")
def get_all_interface_details():
    r = db.get_redis()
    result = _decode_utf8_dict(
            r.hgetall(interfaces_key))

    return Response(
        json.dumps(result),
        mimetype="application/json")


@routes.route("/interfaces/<equipment_name>")
def get_interface_details_for_equipment(equipment_name):
    r = db.get_redis()
    result = {}
    for t in r.hscan_iter(interfaces_key, "{}::*".format(equipment_name)):
        result[t[0].decode("utf8")] = json.loads(t[1])

    return Response(
        json.dumps(result),
        mimetype="application/json")


@routes.route("/interfaces/<equipment_name>/<path:interface>")
def get_interface_details(equipment_name, interface):
    r = db.get_redis()
    return Response(
        r.hget(
            interfaces_key,
            "{}::{}".format(equipment_name, interface)),
        mimetype="application/json")


@routes.route("/equipment-location")
def get_all_equipment_locations():
    r = db.get_redis()
    result = list(
        _decode_utf8_dict(
            r.hgetall(equipment_locations_key)).values())

    return Response(
        json.dumps(result),
        mimetype="application/json")


@routes.route("/equipment-location/<equipment_name>")
def get_equipment_location(equipment_name):
    r = db.get_redis()
    return Response(
        r.hget(equipment_locations_key, equipment_name),
        mimetype="application/json")

#  todo - Add in the routes for the circuit hierarchy, the actual load of the data is already done


# todo - Below are temporary routes that need to be removed

@routes.route("/circuit-sql")
def get_circuits_sql():
    return Response(opsdb._generate_get_circuits_sql(), mimetype="text/plain")


@routes.route("/infinera-circuit-sql", methods=['GET',])
def get_infinera_circuits_sql():
    return Response(opsdb.generate_infinera_sql(), mimetype="text/plain")


@routes.route("/equipment-location-sql", methods=['GET',])
def get_equipment_location_sql():
    return Response(opsdb.equipment_location_query, mimetype="text/plain")